import os
from re import compile

from django.conf import settings
from django.contrib.sites.models import Site
from django.template.context import RequestContext
from django.template.response import HttpResponse
from django.views.generic import TemplateView

from wkhtmltopdf.utils import template_to_temp_file, wkhtmltopdf


class PDFResponse(HttpResponse):
    def __init__(self, content, *args, **kwargs):
        filename = kwargs.pop('filename', None)
        super(PDFResponse, self).__init__(content, 'application/pdf', *args, **kwargs)
        if filename:
            header_content = 'attachment; filename={0}'.format(filename)
            self.__setitem__('Content-Disposition', header_content)


class PdfResponse(PDFResponse):
    def __init__(self, content, filename):
        warning = '''PdfResponse is deprecated in favour of PDFResponse. It will be removed in version 1.'''
        raise PendingDeprecationWarning(warning)
        super(PdfResponse, self).__init__(content, filename)


class PDFTemplateView(TemplateView):
    filename = 'rendered_pdf.pdf'
    footer_template = None
    header_template = None
    orientation = 'portrait'
    margin_bottom = 0
    margin_left = 0
    margin_right = 0
    margin_top = 0
    response = PDFResponse
    _tmp_files = None

    def __init__(self, *args, **kwargs):
        self._tmp_files = []
        super(PDFTemplateView, self).__init__(*args, **kwargs)

    def get(self, request, context_instance=None, *args, **kwargs):
        if request.GET.get('as', '') == 'html':
            return super(PDFTemplateView, self).get(request, *args, **kwargs)

        if context_instance:
            self.context_instance = context_instance
        else:
            self.context_instance = RequestContext(request, self.get_context_data(**kwargs))

        page_path = template_to_temp_file(self.get_template_names(), self.get_context_data(), self.context_instance)
        pdf_kwargs = self.get_pdf_kwargs()
        output = wkhtmltopdf(page_path, **pdf_kwargs)
        if self._tmp_files:
            map(os.remove, self._tmp_files)
        return self.response(output, filename=self.get_filename())

    def get_filename(self):
        return self.filename

    def get_pdf_kwargs(self):
        kwargs = {
            'margin_bottom': self.margin_bottom,
            'margin_left': self.margin_left,
            'margin_right': self.margin_right,
            'margin_top': self.margin_top,
            'orientation': self.orientation,
        }
        if self.header_template:
            kwargs['header_html'] = template_to_temp_file(self.header_template, self.get_context_data(), self.context_instance)
            self._tmp_files.append(kwargs['header_html'])
        if self.footer_template:
            kwargs['footer_html'] = template_to_temp_file(self.footer_template, self.get_context_data(), self.context_instance)
            self._tmp_files.append(kwargs['footer_html'])
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(PDFTemplateView, self).get_context_data(**kwargs)

        match_full_url = compile(r'^https?://')
        if not match_full_url.match(settings.STATIC_URL):
            context['STATIC_URL'] = 'http://' + Site.objects.get_current().domain + settings.STATIC_URL
        if not match_full_url.match(settings.MEDIA_URL):
            context['MEDIA_URL'] = 'http://' + Site.objects.get_current().domain + settings.MEDIA_URL

        return context


class PdfTemplateView(PDFTemplateView): #TODO: Remove this in v1.0
    def as_view(cls, **initkwargs):
        warning = '''PdfTemplateView is deprecated in favour of PDFTemplateView. It will be removed in version 1.'''
        raise PendingDeprecationWarning(warning)
        return super(PdfTemplateView, cls).as_view(**initkwargs)



from django.test import TestCase

from utils import template_to_temp_file

class TestUtils(TestCase):
    def test_template_to_temp_file(self):
        """Should render a template to a temporary file."""
        title = 'A test template.'
        temp_file = template_to_temp_file('sample.html', {'title': title})
        with open(temp_file, 'r') as f:
            saved_content = f.read()
        self.assertTrue(title in saved_content)




from os import fdopen
from subprocess import Popen, PIPE, CalledProcessError
from tempfile import mkstemp

from django.conf import settings
from django.template import loader
from django.utils.encoding import smart_str

WKHTMLTOPDF_CMD = getattr(settings, 'WKHTMLTOPDF_CMD', 'wkhtmltopdf')

def wkhtmltopdf(pages, output=None, **kwargs):
    """
    Converts html to PDF using http://code.google.com/p/wkhtmltopdf/.

    pages: List of file paths or URLs of the html to be converted.
    output: Optional output file path.
    **kwargs: Passed to wkhtmltopdf via _extra_args() (See
              https://github.com/antialize/wkhtmltopdf/blob/master/README_WKHTMLTOPDF
              for acceptable args.)
              Kwargs is passed through as arguments. e.g.:
                  {'footer_html': 'http://example.com/foot.html'}
              becomes
                  '--footer-html http://example.com/foot.html'
              Where there is no value passed, use a blank string. e.g.:
                  {'disable_javascript': ''}
              becomes:
                  '--disable-javascript '

    example usage:
        wkhtmltopdf(html_path="~/example.html",
                    dpi=300,
                    orientation="Landscape",
                    disable_javascript="")
    """

    def _extra_args(**kwargs):
        """Converts kwargs into a string of flags to be passed to wkhtmltopdf."""
        flags = ''
        for k, v in kwargs.items():
            flags += ' --%s %s' % (k.replace('_', '-'), v)
        return flags

    if not isinstance(pages, list):
        pages = [pages]

    kwargs['quiet'] = ''
    args = '%s %s %s %s' % (WKHTMLTOPDF_CMD, _extra_args(**kwargs), ' '.join(pages), output or '-')

    process = Popen(args, stdout=PIPE, shell=True)
    stdoutdata, stderrdata = process.communicate()

    if process.returncode != 0:
        raise CalledProcessError(process.returncode, args)

    if output is None:
        output = stdoutdata

    return output

def template_to_temp_file(template_name, dictionary=None, context_instance=None):
    """
    Renders a template to a temp file, and returns the path of the file.
    """
    file_descriptor, tempfile_path = mkstemp(suffix='.html')
    with fdopen(file_descriptor, 'wt') as f:
        f.write(smart_str(loader.render_to_string(template_name, dictionary=dictionary, context_instance=context_instance)))
    return tempfile_path




from setuptools import setup, find_packages

import wkhtmltopdf


setup(
    name='django-wkhtmltopdf',
    packages=find_packages(),
    include_package_data=True,
    version=wkhtmltopdf.__version__,
    description='Converts html to PDF using http://code.google.com/p/wkhtmltopdf/.',
    long_description=open('README.rst').read(),
    author=wkhtmltopdf.__author__,
    author_email='admin@incuna.com',
    url='http://incuna.com/',
)

